import { Injectable, defineInjectable, Component, ViewContainerRef, Input, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { HttpClient, HttpClientModule } from '@angular/common/http';
import { takeUntil, map, catchError } from 'rxjs/operators';
import { hooks, highlight, languages } from 'prismjs';
import 'prismjs/components/prism-typescript';
import 'prismjs/components/prism-bash';
import { Subject, of } from 'rxjs';
import { FormsModule } from '@angular/forms';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class UtilitiesService {
    constructor() { }
    /* parse html from and return element matched with selector */
    /**
     * @param {?} template
     * @param {?} selector
     * @return {?}
     */
    parseHtml(template, selector) {
        /** @type {?} */
        const parser = new DOMParser();
        /** @type {?} */
        let htmlTemplate = '';
        try {
            htmlTemplate = typeof template !== 'string' ? template['_data'].componentView.component.viewContainerRef['_view'].component.constructor['__annotations__'][0].template : template;
        }
        catch (e) {
            console.warn('Exemplify: couldn\'t retrieve html for "' + selector + '", most likely due to annotations being removed with aot build. Referer to the following issue for more info and workaround:', 'https://github.com/hjalmers/angular-exemplify/issues/15');
        }
        /** @type {?} */
        let output = '';
        /** @type {?} */
        const markupExampleCode = parser.parseFromString(htmlTemplate, 'text/html').querySelectorAll(selector);
        if (markupExampleCode.length === 0 && htmlTemplate !== '') {
            console.warn('Exemplify: no element matched selector, no markup returned.');
        }
        for (let i = 0; i < markupExampleCode.length; i++) {
            output += markupExampleCode[i].outerHTML + '\n';
        }
        return output === '' ? 'no elements matched: "' + selector + '"' : output;
    }
}
UtilitiesService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
/** @nocollapse */
UtilitiesService.ctorParameters = () => [];
/** @nocollapse */ UtilitiesService.ngInjectableDef = defineInjectable({ factory: function UtilitiesService_Factory() { return new UtilitiesService(); }, token: UtilitiesService, providedIn: "root" });

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ExemplifyComponent {
    /**
     * @param {?} viewContainerRef
     * @param {?} utilities
     */
    constructor(viewContainerRef, utilities) {
        this.viewContainerRef = viewContainerRef;
        this.utilities = utilities;
        this._escapeStrings = [];
        this._sources = [];
        this._show = true;
        this._defaultTexts = {
            markup: 'markup',
            copy: 'Copy',
            show: 'Show',
            hide: 'Hide',
            sourceNotFound: 'Source not found',
            copySuccess: 'Code snippet successfully copied to clipboard!',
            copyError: 'Copy to clipboard failed!'
        };
        this._texts = this._defaultTexts;
    }
    /**
     * @return {?}
     */
    get texts() {
        return this._texts;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set texts(value) {
        this._texts = Object.assign({}, this._defaultTexts, value);
    }
    /**
     * @return {?}
     */
    get escapeStrings() {
        return this._escapeStrings;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set escapeStrings(value) {
        this._escapeStrings = value;
    }
    /**
     * @return {?}
     */
    get show() {
        return this._show;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set show(value) {
        this._show = value;
    }
    /**
     * @return {?}
     */
    get sources() {
        return this._sources;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set sources(value) {
        this._sources = value.map((/**
         * @param {?} snippet
         * @return {?}
         */
        snippet => {
            return {
                name: snippet.name || snippet.src.split('/').pop(),
                lang: snippet.lang,
                src: snippet.src,
                selector: snippet.selector
            };
        }));
    }
    /**
     * @return {?}
     */
    get selector() {
        return this._selector;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set selector(value) {
        this._selector = value;
        // get parsed html template from view container reference using selector
        /** @type {?} */
        const parsedHtml = this.utilities.parseHtml(this.viewContainerRef, this.selector);
        this.addSnippet({
            name: this.texts.markup,
            src: this.escapeString(parsedHtml, this.escapeStrings),
            lang: 'markup'
        });
    }
    /* set active snippet */
    /**
     * @param {?} snippet
     * @return {?}
     */
    setActiveSnippet(snippet) {
        this.activeSnippet = Object.assign({}, snippet, { isActive: this.show });
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        // when inputs are changed, update active snippet if no snippet is active
        if (!this.activeSnippet) {
            // set first snippet in array to be active by default
            this.setActiveSnippet(this.sources[0]);
        }
    }
    /* compare options */
    /**
     * @param {?} o1
     * @param {?} o2
     * @return {?}
     */
    compareFn(o1, o2) {
        return o1 && o2 ? o1.name === o2.name : o1 === o2;
    }
    /* prepend snippet to list of snippet */
    /**
     * @private
     * @param {?} snippet
     * @return {?}
     */
    addSnippet(snippet) {
        this.sources = [snippet, ...this.sources];
    }
    /* escape strings, by default angular will turn all directives and attributes into lower case when rendering the html, but we want to show them as they should be entered */
    /**
     * @private
     * @param {?} html
     * @param {?} strings
     * @return {?}
     */
    escapeString(html, strings) {
        // add angular attributes and directives to the list of strings to escape
        /** @type {?} */
        const stringsToEscape = [...strings, '*ngIf', '*ngFor', '*ngPluralCase', '*ngSwitchCase', '*ngSwitchDefault', 'ngClass', 'ngPlural', 'ngStyle', 'ngSwitch', 'ngTemplateOutlet', '[ngIf]', '[ngFor]', '[ngForOf]', '[ngPluralCase]', '[ngSwitchCase]', '[ngSwitchDefault]', 'ngModel'];
        // loop through strings to escape and replace the lowercase version with the escaped version
        stringsToEscape.map((/**
         * @param {?} string
         * @return {?}
         */
        string => {
            /** @type {?} */
            const lower = new RegExp('\\' + string.toLowerCase(), 'g');
            html = html.replace(lower, string).replace('=""', '');
        }));
        return html;
    }
}
ExemplifyComponent.decorators = [
    { type: Component, args: [{
                selector: 'exemplify',
                template: "<ul class=\"nav nav-tabs d-none d-sm-flex\">\n  <li class=\"nav-item\" *ngFor=\"let snippet of sources\">\n    <button class=\"nav-link\" [ngClass]=\"{'active': snippet.name === activeSnippet.name}\"\n            (click)=\"setActiveSnippet(snippet)\">{{snippet.name}}\n    </button>\n  </li>\n</ul>\n<select class=\"custom-select d-sm-none\" [compareWith]=\"compareFn\" [(ngModel)]=\"activeSnippet\">\n  <option *ngFor=\"let snippet of sources\" [ngValue]=\"snippet\">{{snippet.name}}</option>\n</select>\n<code-snippet [snippet]=\"activeSnippet\" [texts]=\"texts\"></code-snippet>\n",
                styles: [":host{display:block;position:relative;margin-top:1rem}:host:hover ::ng-deep.btn-group{opacity:1;-webkit-transform:translate3d(0,0,0);transform:translate3d(0,0,0)}:host:hover ::ng-deep.btn-group .btn.not-active{text-decoration:underline}.nav-link:not(.active){border-bottom:inherit}@media only screen and (max-width:576px){.custom-select{padding-right:114px;background-position-x:calc(100% - 114px - 8px)}}"]
            }] }
];
/** @nocollapse */
ExemplifyComponent.ctorParameters = () => [
    { type: ViewContainerRef },
    { type: UtilitiesService }
];
ExemplifyComponent.propDecorators = {
    texts: [{ type: Input }],
    escapeStrings: [{ type: Input }],
    show: [{ type: Input }],
    sources: [{ type: Input }],
    selector: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @return {?}
 */
function _window() {
    // return the global native browser window object
    return window;
}
class WindowRef {
    constructor() { }
    /**
     * @return {?}
     */
    get nativeWindow() {
        return _window();
    }
}
WindowRef.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
/** @nocollapse */
WindowRef.ctorParameters = () => [];
/** @nocollapse */ WindowRef.ngInjectableDef = defineInjectable({ factory: function WindowRef_Factory() { return new WindowRef(); }, token: WindowRef, providedIn: "root" });

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// Prism tries to highlight the whole document on DOMContentLoad.
// Unfortunately with webpack the only way of disabling it
// is by simply forcing it to highlight no elements -> []
hooks.add('before-highlightall', (/**
 * @param {?} env
 * @return {?}
 */
env => {
    env['elements'] = [];
}));
class CodeHighlightService {
    /**
     * @param {?} code
     * @param {?} lang
     * @return {?}
     */
    highlight(code, lang) {
        try {
            return highlight(code.trim(), languages[lang]);
        }
        catch (error) {
            console.log(error);
        }
    }
}
CodeHighlightService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
/** @nocollapse */ CodeHighlightService.ngInjectableDef = defineInjectable({ factory: function CodeHighlightService_Factory() { return new CodeHighlightService(); }, token: CodeHighlightService, providedIn: "root" });

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CodeSnippetComponent {
    /**
     * @param {?} _codeHighlight
     * @param {?} http
     * @param {?} windowRef
     * @param {?} utilities
     */
    constructor(_codeHighlight, http, windowRef, utilities) {
        this._codeHighlight = _codeHighlight;
        this.http = http;
        this.windowRef = windowRef;
        this.utilities = utilities;
        this.isActive = true;
        this.showCopyNotice = false;
        this.$unsubscribe = new Subject();
        this._defaultTexts = {
            markup: 'markup',
            copy: 'Copy',
            show: 'Show',
            hide: 'Hide',
            sourceNotFound: 'Source not found',
            copySuccess: 'Code snippet successfully copied to clipboard!',
            copyError: 'Copy to clipboard failed!'
        };
        this._texts = this._defaultTexts;
        /**
         * Copy to clipboard
         */
        this.copyToClipboard = (/**
         * @param {?} text
         * @return {?}
         */
        function (text) {
            if (this.window.clipboardData && this.window.clipboardData.setData) {
                // IE specific code path to prevent textarea being shown while dialog is visible.
                return this.window.clipboardData.setData('Text', text);
            }
            else if (document.queryCommandSupported && document.queryCommandSupported('copy')) {
                /** @type {?} */
                const textarea = document.createElement('textarea');
                textarea.textContent = text;
                textarea.style.position = 'fixed'; // Prevent scrolling to bottom of page in MS Edge.
                document.body.appendChild(textarea);
                textarea.select();
                try {
                    /** @type {?} */
                    const copy = document.execCommand('copy');
                    this.copyNotice = this.texts.copySuccess;
                    this.showCopyNotice = true;
                    return copy;
                }
                catch (ex) {
                    console.warn('Copy to clipboard failed.', ex);
                    this.copyNotice = this.texts.copyError;
                    this.showCopyNotice = true;
                    return false;
                }
                finally {
                    document.body.removeChild(textarea);
                    setTimeout((/**
                     * @return {?}
                     */
                    () => this.showCopyNotice = false));
                }
            }
        });
        this.window = this.windowRef.nativeWindow;
    }
    /**
     * @return {?}
     */
    get texts() {
        return this._texts;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set texts(value) {
        this._texts = Object.assign({}, this._defaultTexts, value);
    }
    /**
     * @return {?}
     */
    get snippet() {
        return this._snippet;
    }
    /**
     * @param {?} snippet
     * @return {?}
     */
    set snippet(snippet) {
        // guess file extension based on source file, note this won't work when source is inline or when raw-loader is used
        /** @type {?} */
        const e = snippet.src.split('.');
        /** @type {?} */
        const extension = e[e.length - 1];
        this._snippet = Object.assign({}, snippet, { code: snippet.src.substring(0, 4) === 'http' ? this.getSourceCode(snippet.src, snippet.selector) : of(snippet.src), lang: snippet.lang || (extension.length > 4 ? 'markup' : extension) // if lang is omitted use extension if length is not greater than 4 otherwise default to markup
         });
        // when snippet has loaded highlight the code and store the string representation too (needed for copying etc).
        this.snippet.code
            .pipe(takeUntil(this.$unsubscribe))
            .subscribe((/**
         * @param {?} code
         * @return {?}
         */
        code => {
            this.highlight(code, this.snippet.lang);
            this.code = code; // store raw string representation
        }));
        this.toggleState(this.snippet.isActive !== false);
    }
    /**
     * Highlight, highlight code and update value of parsed snippet
     * @param {?} code
     * @param {?} lang
     * @return {?}
     */
    highlight(code, lang) {
        this.parsedSnippet = this._codeHighlight.highlight(code, lang);
    }
    /**
     * Get source code
     * @private
     * @param {?} url
     * @param {?=} selector
     * @return {?}
     */
    getSourceCode(url, selector) {
        return this.http.get(url, { responseType: 'text' })
            .pipe(map((/**
         * @param {?} data
         * @return {?}
         */
        data => {
            if (selector) {
                return this.utilities.parseHtml(data, selector);
            }
            else {
                return data;
            }
        }), (/**
         * @param {?} error
         * @return {?}
         */
        error => console.log(error))), catchError((/**
         * @param {?} error
         * @return {?}
         */
        (error) => {
            if (error.error instanceof ErrorEvent) {
                // a client-side or network error occurred
                console.error('An error occurred:', error.error.message);
            }
            return of(this.texts.sourceNotFound);
        })));
    }
    /**
     * Toggle state for code snippet toolbar
     * @param {?=} state
     * @return {?}
     */
    toggleState(state) {
        this.isActive = state || !this.isActive;
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.$unsubscribe.next();
        this.$unsubscribe.complete();
    }
}
CodeSnippetComponent.decorators = [
    { type: Component, args: [{
                selector: 'code-snippet',
                template: "<div class=\"btn-group\" role=\"group\" aria-label=\"Code snippet toolbar\" *ngIf=\"code\">\n  <button class=\"btn btn-link\" (click)=\"toggleState()\" [ngClass]=\"{'not-active':!isActive}\">{{isActive ? texts.hide:texts.show}}</button>\n  <button class=\"btn btn-link\" (click)=\"copyToClipboard(code)\">{{texts.copy}}</button>\n</div>\n<div class=\"snippet-wrapper exemplify-backdrop\" [ngClass]=\"{'exemplify-hidden': !showCopyNotice }\" (dblclick)=\"copyToClipboard(code)\">\n  <pre class=\"language-{{snippet?.lang}}\" *ngIf=\"isActive\">\n    <code class=\"language-{{snippet?.lang}}\" [innerHTML]=\"parsedSnippet\"></code>\n  </pre>\n  <div class=\"copy-notice\" [ngClass]=\"{'exemplify-hidden': !showCopyNotice }\">\n    <span>{{copyNotice}}</span>\n  </div>\n</div>\n\n",
                styles: [".btn-group{position:absolute;top:0;right:0}@media only screen and (min-width:576px){.btn-group{opacity:0;margin-top:2px;-webkit-transform:translate3d(0,10px,0);transform:translate3d(0,10px,0);transition:opacity .4s ease-in-out,transform .4s ease-out,-webkit-transform .4s ease-out}}@media only screen and (max-width:576px){.btn-group{width:114px;height:calc(2.25rem + 2px)}.btn-group .btn{height:100%;padding-right:.5rem;padding-left:.5rem}}.snippet-wrapper{position:relative}.snippet-wrapper.exemplify-backdrop::after{content:'';display:block;height:100%;width:100%;top:0;left:0;position:absolute;background:rgba(0,0,0,.75);z-index:0;opacity:1}.copy-notice{position:absolute;height:100%;width:100%;top:0;text-align:center;color:#fff;font-size:1.25rem;display:flex;justify-content:center;align-items:center;max-height:300px;z-index:1;opacity:1}.exemplify-hidden.copy-notice,.exemplify-hidden.exemplify-backdrop::after{transition:opacity .6s ease-out 1.4s,z-index 1ms 1.4s,height 1ms 2s;opacity:0;z-index:0;height:0}"]
            }] }
];
/** @nocollapse */
CodeSnippetComponent.ctorParameters = () => [
    { type: CodeHighlightService },
    { type: HttpClient },
    { type: WindowRef },
    { type: UtilitiesService }
];
CodeSnippetComponent.propDecorators = {
    texts: [{ type: Input }],
    snippet: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ExemplifyModule {
}
ExemplifyModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule, FormsModule, HttpClientModule],
                declarations: [ExemplifyComponent, CodeSnippetComponent],
                exports: [ExemplifyComponent]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { ExemplifyComponent, ExemplifyModule, UtilitiesService as ɵa, CodeSnippetComponent as ɵb, CodeHighlightService as ɵc, WindowRef as ɵd };
//# sourceMappingURL=angular-exemplify.js.map
